<?php

/*
FlickrMovr's XML database

Functions:
- updateLastDownloadFinished
- createDB
- loadDB
- saveDB
- updateSet
- updateSetAttributes
- getPhotoLinksInSet
- getPhoto
- photoLinkExists
- updatePhotoAttributes
*/

class FlickrMovrDOMDocument extends DOMDocument {

	private $root=null;
	private $fileName=null;

	/*
	Constructor

	handles initiation of xml database
	and sets global last_download_started
	attribute
	*/

	public function __construct($fileName) {
		parent::__construct();

		$this->setFileName($fileName);

		/*
		format XML
		*/

		$this->formatOutput = true;
		$this->preserveWhiteSpace = false;

		/*
		load flickrmovr db
		*/

		if (file_exists($this->getFileName()))
			$this->loadDB();

		/*
		if it does not exist, create new db
		*/

		else
			$this->createDB();

		/*
		remove attribute last_download_finished
		*/

		$this->root->removeAttribute('last_download_finished');

		/*
		update attribute last_download_started
		*/

		$this->root->setAttribute('last_download_started', time());

		/*
		set user attributes
		*/

		$this->root->setAttribute('user_name', FLICKR_USER_NAME);

		$this->root->setAttribute('nsid', NSID);

		/*
		save set information to
		xml database
		*/

		$this->saveDB();
	}

	/*
	setters and getters
	*/

	private function setFileName($fileName) {
		$this->fileName=$fileName;
	}

	private function getFileName() {
		return $this->fileName;
	}

	private function setRoot($root) {
		$this->root=$root;
	}

	private function getRoot() {
		return $this->root;
	}

	/*
	updates global last download finished attribute
	*/

	public function updateLastDownloadFinished() {
		$this->root->setAttribute('last_download_finished', time());
		/*
		save set information to
		xml database
		*/

		$this->saveDB();
	}

	/*
	constructs a new FlickMovr database
	*/

	private function createDB() {

		/*
		include link to xslt
		*/

		$xslt = $this->createProcessingInstruction('xml-stylesheet', 'type="text/xsl" href="flickrmovr.xsl"');
		$this->appendChild($xslt);

		/*
		create and append root element
		*/

		$this->setRoot($this->createElement("flickrmovr"));
		$this->appendChild($this->getRoot());

		/*
		append comment
		*/

		$created_by=$this->createComment("Created by FlickrMovr &copy; Dr. Bernhard Riedl http://www.bernhard-riedl.com/");
		$this->appendChild($created_by);

		/*
		save set information to
		xml database
		*/

		$this->saveDB();
	}

	/*
	loads an existing FlickrMovr database
	*/

	private function loadDB() {

		/*
		load xml database
		*/

		$this->load(FLICKR_XML);

		/*
		retrieve root-node
		*/

		$nodeRoot=$this->getElementsByTagName("flickrmovr");
		if ($nodeRoot->length == 1) {
			$this->setRoot($nodeRoot->item(0));
		}

		else {
			throw new Exception("Failed to load XML database");
		}
	}

	/*
	saves FlickrMovr database
	*/

	private function saveDB() {
		$this->save($this->getFileName());
	}

	/*
	appends or updates a complete
	set-tree (including all photos
	and photo_links) in the database
	*/

	public function updateSet($setXML) {
		$this->getRoot()->appendChild($setXML);

		/*
		save set information to
		xml database
		*/

		$this->saveDB();
	}

	/*
	updates parameters of an existing photoset
	from the FlickrMovr database
	if the current set does not exist, it will
	initiate the creation

	returns DOMElement setXML
	*/

	public function updateSetAttributes($set) {
		$setID='set_'.$set->getID();

		$setXML=$this->getElementById($setID);

		/*
		if photoset does not exist
		create new node
		*/

		if (empty($setXML)) {
			$setXML = $this->createElement("set");
			$setXML->setAttribute('xml:id', $setID);
		}

		/*
		append/update set attributes
		*/

		$setXML->setAttribute('title', $set->getTitle());
		$setXML->setAttribute('description', $set->getDescription());
		$setXML->setAttribute('directory', $set->getDirectoryName());

		/*
		append/update set in db
		*/

		$this->getRoot()->appendChild($setXML);

		/*
		save set information to
		xml database
		*/

		$this->saveDB();

		return $setXML;
	}

	/*
	returns the photo-links of
	a certain photo-set

	photo links hold ids of
	photos which are already
	stored in another subset
	*/

	public function getPhotoLinksInSet($setXML) {

		/*
		retrieve photo_links of current set
		if the node does not exist,
		we append it
		*/

		$setLinksXML=$setXML->getElementsByTagName("photo_links");

		if ($setLinksXML->length == 1) {
			$setLinksXML = $setLinksXML->item(0);
		}

		/*
		in case we didn't find the node,
		create it
		*/

		else {
			$setLinksXML=$this->createElement("photo_links");

			/*
			append photo_links child to set
			*/

			$setXML->appendChild($setLinksXML);

			/*
			save set information to
			xml database
			*/

			$this->saveDB();
		}

		return $setLinksXML;

	}

	/*
	returns a DOMElement node holding a photo
	*/

	public function getPhoto($photoID) {
		return $this->getElementById('photo_'.$photoID);
	}

	/*
	check if photo_link already exists
	in a particular set
	*/

	public function photoLinkExists($photo, $setLinksXML) {
		$photo_links=$setLinksXML->getElementsByTagName("photo_link");

		/*
		does the photo_link exist already?
		*/

		foreach ($photo_links as $photo_link) {
			if ($photo_link->getAttribute('id')==('photo_'.$photo->getID())) {
				return true;
			}
		}

		return false;
	}

	/*
	updates parameters of an existing photo
	in the FlickrMovr database

	if the current photo does not exist, we will
	initiate the creation

	also handles photo_links (photos which
	are already in another set and are only
	stored by the usage of their particular id)
	*/

	public function updatePhotoAttributes($photo, $setXML, $setLinksXML) {
		$photoXML=$this->getPhoto($photo->getID());

		/*
		we need to create a new photo element
		*/

		if (empty($photoXML)) {
			$photoXML = $this->createElement("photo");
			$photoXML->setAttribute('xml:id', 'photo_'.$photo->getID());
		}

		/*
		we update an existing photo element
		*/

		else {

			/*
			check if setXML is our parent,
			if not, we have to handle a photo_link
			*/

			if (!$photoXML->parentNode->isSameNode($setXML)) {

				/*
				if photo-link does not exist
				create and append new element
				*/

				if (!$this->photoLinkExists($photo, $setLinksXML)) {
					$photoXML=$this->createElement("photo_link");
					$photoXML->setAttribute('id', 'photo_'.$photo->getID());
					$setLinksXML->appendChild($photoXML);

					/*
					save set information to
					xml database
					*/

					$this->saveDB();
				}

			/*
			photo-link handeled
			*/

			return true;

			}

			/*
			no photo-link -> progress with
			standard photo handling
			*/
		}

		/*
		set or update photo attributes
		*/

		$photoXML->setAttribute("title", $photo->getTitle());
		$photoXML->setAttribute("description", $photo->getDescription());
		$photoXML->setAttribute("date_taken", $photo->getDateTaken());
		$photoXML->setAttribute("last_update", $photo->getLastUpdate());

		$photoXML->setAttribute("flickr_link", $photo->getFlickrLink());

		/*
		retrieve location of current photo
		if the node does not exist,
		we append it
		*/

		$location_xml=$photoXML->getElementsByTagName("location");

		if ($location_xml->length == 1) {
			$location_xml = $location_xml->item(0);
		}

		/*
		in case we didn't find the node,
		create it
		*/

		else {
			$location_xml=$this->createElement("location");
		}

		$location_xml->setAttribute("latitude", $photo->getLatitude());
		$location_xml->setAttribute("longitude", $photo->getLongitude());
		$photoXML->appendChild($location_xml);

		/*
		remove current tag-subtrees from photo
		*/

		$oldTags=$photoXML->getElementsByTagName("tags");
		foreach ($oldTags as $oldTag) {
			$photoXML->removeChild($oldTag);
		}

		/*
		create new tags element
		*/

		$tagsXML=$this->createElement("tags");

		/*
		create tag element in tags sub-tree
		and set the flickr tag-name as name attribute
		*/

		foreach ($photo->getTags() as $tag) {
			$tagXML=$this->createElement("tag");
			$tagXML->setAttribute("name", $tag);
			$tagsXML->appendChild($tagXML);
		}

		/*
		append tags to photo
		*/

		$photoXML->appendChild($tagsXML);

		/*
		append photo to set
		*/

		$setXML->appendChild($photoXML);

		/*
		save set information to
		xml database
		*/

		$this->saveDB();

		return true;
	}

}
