<?php

/*
This class processes a flickr-set
and its meta-tags

Functions:
- setMeta
- makeDirectory (only log-message on error)
- makeDirectories
- processPhotos
*/

class FlickrMovrSet {

	private $id=-1;
	private $title="";
	private $description="";
	private $directoryName="";

	private $flickrXML=null;
	private $xml=null;
	private $photoLinksXML=null;

	/*
	Constructor
	*/

	public function __construct($flickrXML, $meta) {

		$this->setFlickrXML($flickrXML);

		$this->setMeta($meta);

		$this->setDirectoryName(substr(Utils::remove_non_path_characters($this->getTitle()), 0, MAX_PATH_LENGTH));

		/*
		create necessary directories
		*/

		$this->makeDirectories();

		/*
		get/append instance of XML node
		*/

		if (CREATE_XML) {
			$this->setXML($this->getFlickrXML()->updateSetAttributes($this));
			$this->setPhotoLinksXML($this->getFlickrXML()->getPhotoLinksInSet($this->getXML()));
		}

		/*
		start processing of flickr photos
		*/

		$this->processPhotos();
	}

	/*
	setters and getters
	*/

	private function setFlickrXML($flickr_xml) {
		$this->flickr_xml=$flickr_xml;
	}

	public function getFlickrXML() {
		return $this->flickr_xml;
	}

	private function setID($id) {
		$this->id=$id;
	}

	public function getID() {
		return $this->id;
	}

	private function setTitle($title) {
		$this->title=$title;
	}

	public function getTitle() {
		return $this->title;
	}

	private function setDescription($description) {
		$this->description=$description;
	}

	public function getDescription() {
		return $this->description;
	}

	private function setDirectoryName($directoryName) {
		$this->directoryName=$directoryName;
	}

	public function getDirectoryName() {
		return $this->directoryName;
	}

	public function getDirectory() {
		return PHOTO_DIR.$this->getDirectoryName();
	}

	public function getThumbnailsDirectory() {
		return $this->getDirectory().DIRECTORY_SEPARATOR.THUMBNAILS_DIR;
	}

	private function setXML($xml) {
		$this->xml=$xml;
	}

	public function getXML() {
		return $this->xml;
	}

	private function setPhotoLinksXML($photoLinksXML) {
		$this->photoLinksXML=$photoLinksXML;
	}

	public function getPhotoLinksXML() {
		return $this->photoLinksXML;
	}

	/*
	set the photo-set's meta-data
	*/

	private function setMeta($meta) {
		$this->setID($meta['id']);

		if (array_key_exists('title', $meta))
			$this->setTitle(html_entity_decode($meta['title']['_content'], ENT_QUOTES));

		if (array_key_exists('description', $meta))
			$this->setDescription(html_entity_decode($meta['description']['_content'], ENT_QUOTES));
	}

	/*
	make sets photos/thumbnails directory
	*/

	private function makeDirectory($directory, $type) {
		if (!file_exists($directory))
			if(!@mkdir($directory))
				Utils::echo_log('Failed to create '.$type.' directory '.$directory, -1);
	}

	/*
	creates set's photos and thumbnails directoy
	*/

	private function makeDirectories() {
		Utils::echo_log('Process set '.$this->getDirectoryName());

		/*
		create photo-set's directory
		if user decided to download
		photos or thumbnails
		*/

		if (DOWNLOAD_PHOTOS || DOWNLOAD_THUMBNAILS)
			$this->makeDirectory($this->getDirectory(), 'photos');

		/*
		create photo-set's thumbnail directory
		*/

		if (DOWNLOAD_THUMBNAILS)
			$this->makeDirectory($this->getThumbnailsDirectory(), 'thumbnails');
	}

	/*
	initiate handling of photos
	*/

	private function processPhotos() {
		$query_params=array('photoset_id' => $this->getID());

		/*
		save same bandwith, as we only need the
		last_update in case we do an
		incremental backup
		*/

		if (INCREMENTAL_BACKUP)
			$query_params['extras'] = 'last_update';

		$rsp_obj=Utils::query_flickr_api('flickr.photosets.getPhotos', $query_params);

		if ($rsp_obj) {
			foreach($rsp_obj['photoset']['photo'] as $photo) {
				/*
				process all photos in set
				*/

				if (array_key_exists('id', $photo) && (!INCREMENTAL_BACKUP || (INCREMENTAL_BACKUP && array_key_exists('lastupdate', $photo)))) {

					$lastUpdate=0;
					if (INCREMENTAL_BACKUP)
						$lastUpdate=$photo['lastupdate'];

					$flickrMovrPhoto=new FlickrMovrPhoto($photo['id'], $lastUpdate, $this);
				}

				else
					Utils::echo_log('Failed flickr query for single photo.', -1);
			}

			/*
			update set and all childs
			(photos, photo-links, etc.)
			in the database
			*/

			if (CREATE_XML)
				$this->getFlickrXML()->updateSet($this->getXML());

		}

		else
			Utils::echo_log('Failed flickr query', -1);

	}

}
