<?php

/*
FlickrMovr's utils class

Static Functions:
- append_file
- define_constants_create_file
- load_config
- check_api_vars
- output_html_stub
- remove_non_path_characters
- query_flickr_api
- exec_exif_tool
- echo_log
- cURLcheckBasicFunctions
- cURL
- cURLdownload
- authorize_private
- query_nsid
*/

class Utils {

	/*
	creates a textfile and writes one line
	if file exists appends to file
	*/

	static function append_file($file_name, $file_line) {
		$fh = @fopen($file_name, 'a');

		if ($fh) {
			fwrite($fh, $file_line);
			fclose($fh);
		}
	}

	/*
	creates a textfile which holds the
	php-code to define the handed over
	constants;
	also defines all handed over
	constants on-the-fly
	*/

	static function define_constants_create_file($file_name, $file_description, $constants) {
		$fh = @fopen($file_name, 'w');

		if ($fh) {
			fwrite($fh, '<?php ');

			foreach ($constants as $key => $constant) {
				if (!defined($key))
					define($key, $constant);

				fwrite($fh, 'define("'.$key.'", "'.$constant.'");');
			}

			fclose($fh);
		}

		else
			throw new Exception('Sorry, cannot write '.$file_description.'-file ('.$file_name.')!');
	}

	/*
	loads config-file
	which is either passed via
	command-line parameter
	or fallback to config.php
	*/

	static function load_config() {
		global $argv;

		$config_file=(!empty($argv) && sizeof($argv)==2 && !empty($argv[1])) ? $argv[1] : 'config.php';

		if (file_exists($config_file))
			require($config_file);
		else
			throw new Exception('Configuration file '.$config_file.' does not exist!');
	}

	/*
	check if api-parameters have been set
	*/

	static function check_api_vars() {
		if (!defined('API_KEY') || !defined('API_KEY_SECRET'))
			throw new Exception('Please enter your API key details in the config file!');
	}

	/*
	creates a html-file
	for authorization purposes
	*/

	static function output_html_stub($message) { ?><!DOCTYPE html><html>
		<head>
 			<meta charset="UTF-8" />
			<title>FlickrMovr Authorization</title>

			<style>
				.center {
					position: absolute;
					top: 50%;
					left: 50%;
					transform: translateX(-50%) translateY(-50%);
				}
			</style>
		</head>

		<body>

		<div class="center"><?php echo($message); ?></div>

		</body>

		</html><?php

		exit;
	}

	/*
	removes characters which are
	not allowed in directory names
	*/

	static function remove_non_path_characters($path) {
		$path=str_replace(' ', '_', $path);
		$path = preg_replace("/[^a-zA-Z0-9_-]/", "", $path);

		return $path;
	}

	/*
	returns results from a flickr soap query
	with php-json
	https://www.flickr.com/services/api/
	*/

	static function query_flickr_api($method, $query_var="", $query_var_value="") {
		$url='https://api.flickr.com/services/rest/';

		$query_params=array();

		/*
		do we handle an array?
		*/

		if (is_array($query_var))
			$query_params=array_merge($query_params, $query_var);

		/*
		or a single parameter?
		*/

		else if (!empty($query_var))
			$query_params[$query_var]=$query_var_value;

		$query_params['method']=$method;
		$query_params['format']='php_serial';

		$oauth_nonce=md5(implode(':', array(
			API_KEY,
			FLICKR_USER_NAME,
			microtime()
		)));

		$oauth_params=array (
			'oauth_nonce' => $oauth_nonce,
			'oauth_timestamp' => time(),
			'oauth_consumer_key' => API_KEY,
			'oauth_signature_method' => 'HMAC-SHA1',
			'oauth_version' => '1.0'
		);

		$signature_key=API_KEY_SECRET;

		/*
		include oauth_token in query
		and extend signature_key with
		oauth_token_secret
		*/

		if (READ_PRIVATE_PHOTOS && defined('OAUTH_TOKEN') && defined('OAUTH_TOKEN_SECRET')) {
			$signature_key.='&'.OAUTH_TOKEN_SECRET;
			$oauth_params['oauth_token']=OAUTH_TOKEN;
		}

		$params=array_merge($query_params, $oauth_params);

		ksort($params, SORT_STRING);

		$keyValuePairs=array();

		foreach ($params as $k => $v)
			array_push($keyValuePairs, rawurlencode($k)."=".rawurlencode($v));

		$base_string_components=array(
			rawurlencode('POST'),
			rawurlencode($url),
			rawurlencode(implode('&', $keyValuePairs))
		);

		$base_string=implode('&', $base_string_components);

		$params['oauth_signature']=base64_encode(hash_hmac('sha1', $base_string, $signature_key, true));

		$signature_key=null;

		$rsp = Utils::cURL($url, $params);

		$rsp_obj=array('stat' => 'error');

		if ($rsp)
			$rsp_obj = @unserialize($rsp);

		if ($rsp_obj['stat'] == 'ok')
			return $rsp_obj;
		else
			return false;
	}

	/*
	executes ExifTool with simple error-handling
	(1) try with default parameters and
	ignore minor errors (-m) if
	IGNORE_MINOR_EXIF_ERROR has been set to true
	(2) if CORRECT_BROKEN_EXIF_TAGS has been
	set to true on error create temporary
	copy of photo and try to rewrite
	EXIF tags (ExifTool option -unsafe)
	*/

	static function exec_exif_tool($cmd, $photo_file) {

		/*
		remove line-breaks to avoid exec error
		*/

		$br_variants=array("\n", "<br>", "<br />", "<br/>");
		$cmd=str_replace($br_variants, ' ', $cmd);

		/*
		ignore minor errors if option
		is set to true
		*/

		$minor_errors_flag=" ";
		if (IGNORE_MINOR_EXIF_ERRORS)
			$minor_errors_flag=" -m ";

		$log_messages=array();
		$log_ret_val=0;

		@exec(EXIF_TOOL.$minor_errors_flag.$cmd.$photo_file, $log_messages, $log_ret_val);
		$log_line=implode(" ", $log_messages);

		/*
		if we received an error
		we try to remove all EXIF-tags from
		a temporary copy of the photo
		and retrieve all readable tags
		from the original file
		*/

		if (CORRECT_BROKEN_EXIF_TAGS && $log_ret_val>0) {

			$exif_tool_tmp=$photo_file.'_exiftool_tmp';

			/*
			delete temp file
			*/

			if (file_exists($exif_tool_tmp))
				@unlink($exif_tool_tmp);

			$repair_photo_file=$photo_file.'_repair';

			/*
			create temporary copy of photo file
			*/

			@copy($photo_file, $repair_photo_file);

			/*
			provide clean EXIF-database in temp file
			*/

			$log_messages=array();

			$test=@exec(EXIF_TOOL."-all:all= ".$repair_photo_file, $log_messages, $log_ret_val);

			$log_line='first try: '.$log_line.' || try to clean EXIF tags : '.implode(" ", $log_messages);

			/*
			if we didn't receive an error
			try to copy all readable EXIF-tags
			from original photo to temporary file
			*/

			if ($log_ret_val==0) {
				$log_messages=array();

				@exec(EXIF_TOOL."-tagsfromfile ".$photo_file." -all:all --MakerNotes ".$repair_photo_file, $log_messages, $log_ret_val);

				$log_line.=' || try to copy EXIF tags from original '.implode(" ", $log_messages);

				/*
				if we didn't receive an error
				execute original exif command
				*/

				if ($log_ret_val==0) {

					$log_messages=array();

					@exec(EXIF_TOOL.$minor_errors_flag.$cmd.$repair_photo_file, $log_messages, $log_ret_val);

					$log_line.=' || second try : '.implode(" ", $log_messages);

					/*
					overwrite original file with
					temporary file if we did't
					receive an error
					*/

					if ($log_ret_val==0) {
						@copy($repair_photo_file, $photo_file);
					}

				}
			}

			/*
			remove temporary file
			*/

			if (file_exists($repair_photo_file))
				@unlink($repair_photo_file);

		}

		return $log_line;
	}

	/*
	shows log messages on screen
	and optionally appends log message to log file
	*/

	static function echo_log($message, $error=0) {
		$log_line=date('Y-m-d H:i:s')." ";

		/*
		determine the log line's prefix
		*/

		if ($error==0)
			$log_line.="INFO";
		else if ($error==-1)
			$log_line.="ERROR";
		else if ($error==-2)
			$log_line.="EXTERNAL";
		else if ($error==-3)
			$log_line.="FATAL ERROR";

		/*
		set error-constant
		*/

		if ($error<0 && !defined('ERROR'))
			define('ERROR', true);

		/*
		append message
		*/

		$log_line.=" ".$message."\n";

		/*
		output message to screen
		*/

		if (defined('SHOW_LOG_ON_SCREEN') && SHOW_LOG_ON_SCREEN)
			echo($log_line);

		/*
		output message to file
		*/

		if (defined('LOG_FILE') && WRITE_LOG_TO_FILE)
			Utils::append_file(LOG_FILE, $log_line);
	}

	/*
	checks if curl library is installed and enabled

	original version by jlee8df at gmail dot com
	http://php.net/manual/en/curl.examples.php
	*/

	static function cURLcheckBasicFunctions() {
		if(!function_exists("curl_init") || !function_exists("curl_setopt") || !function_exists("curl_exec") || !function_exists("curl_error") || !function_exists("curl_close"))
			throw new Exception('cURL library is not installed!');
	}

	/*
	retrieve data via curl
	*/

	static function cURL($url, $args=array()) {
		$ch=curl_init();

		curl_setopt($ch, CURLOPT_PORT, 443);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_FAILONERROR, true);
		curl_setopt($ch, CURLOPT_HEADER, false);

		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $args);

		$data=curl_exec($ch);
		$error=curl_error($ch);

		if ($error>0)
			return false;

		curl_close($ch);

		return $data;
	}

	/*
	Downloads a file to TEMP_DOWNLOAD_FILE
	and renames the temporary file to the desired
	filename after successful download

	original version by jlee8df at gmail dot com
	http://php.net/manual/en/curl.examples.php
	*/

	static function cURLdownload($url, $file) {
		$ch = @curl_init();

		if($ch) {
			$fp = @fopen(TEMP_DOWNLOAD_FILE, "w");

			if($fp) {
				curl_setopt($ch, CURLOPT_TIMEOUT, 30);

				if(!curl_setopt($ch, CURLOPT_URL, $url)) {
					fclose($fp);
					curl_close($ch);
					return false;
				}

				if(!curl_setopt($ch, CURLOPT_FILE, $fp))
					return false;

				if(!curl_setopt($ch, CURLOPT_HEADER, false))
					return false;

				/*
				download file
				*/

				if(!@curl_exec($ch))
					return false;

				curl_close($ch);
				fclose($fp);

				/*
				copy temp file to desired filename
				*/

				@copy(TEMP_DOWNLOAD_FILE, $file);

				return "$file [$url]";
			}

			else return false;
		}

		else return false;
	}

	/*
	authorize for private photos
	*/

	static function authorize_private() {
		$oauth_token_file=PHOTO_DIR.'oauth_token.php';

		/*
		define OAUTH_TOKEN and
		OAUTH_TOKEN_SECRET from file
		*/

		if (file_exists($oauth_token_file))
			include($oauth_token_file);

		/*
		check if constants have been set
		*/

		if (!defined('OAUTH_TOKEN') || !defined('OAUTH_TOKEN_SECRET'))
			throw new Exception('Authentication token could not be retrieved. - Did you authorize FlickrMovr? - Please call authorize.php, which needs to be hosted on a webserver, from your browser!');

		return true;
	}

	/*
	query for flickr nsid and store in constant
	return false on error
	*/

	static function query_nsid() {
		Utils::echo_log('Query flickr for user-id');

		$rsp_obj=Utils::query_flickr_api('flickr.people.findByUsername', 'username', FLICKR_USER_NAME);

		if ($rsp_obj){
			if (array_key_exists('user', $rsp_obj)) {
				if (array_key_exists('nsid', $rsp_obj['user']) && array_key_exists('username', $rsp_obj['user'])) {
					if (array_key_exists('_content', $rsp_obj['user']['username'])) {
						$user_name=$rsp_obj['user']['username'];
						if ($user_name['_content']==FLICKR_USER_NAME) {
							define('NSID', $rsp_obj['user']['nsid']);
							return NSID;
						}
					}
				}
			}

		}

		return false;

	}

}
